@@ -8,13 +8,14 @@ from pydantic import BaseModel, BaseSettings, validator
 from pydantic.env_settings import InitSettingsSource
 from ruamel.yaml import YAML
 
+from gto.constants import name_regexp
 from gto.exceptions import (
     UnknownStage,
     UnknownType,
     ValidationError,
     WrongConfig,
 )
-from gto.ext import Enrichment, find_enrichment_types, find_enrichments
+from gto.ext import EnrichmentReader, find_enrichment_types, find_enrichments
 
 yaml = YAML(typ="safe", pure=True)
 yaml.default_flow_style = False
@@ -23,7 +24,7 @@ CONFIG_FILE_NAME = ".gto"
 
 
 def check_name_is_valid(name):
-    return bool(re.match(r"[a-z][a-z0-9-/]*[a-z0-9]$", name))
+    return bool(re.search(name_regexp, name))
 
 
 def assert_name_is_valid(name):
@@ -38,7 +39,7 @@ class EnrichmentConfig(BaseModel):
     type: str
     config: Dict = {}
 
-    def load(self) -> Enrichment:
+    def load(self) -> EnrichmentReader:
         return find_enrichment_types()[self.type](**self.config)
 
 
@@ -67,7 +68,7 @@ class NoFileConfig(BaseSettings):
             raise UnknownStage(name, self.STAGES)
 
     @property
-    def enrichments(self) -> Dict[str, Enrichment]:
+    def enrichments(self) -> Dict[str, EnrichmentReader]:
         res = {e.source: e for e in (e.load() for e in self.ENRICHMENTS)}
         if self.AUTOLOAD_ENRICHMENTS:
             return {**find_enrichments(), **res}
